@@ -44,7 +44,9 @@ static R_CallMethodDef CallEntries[] = {
     {"mer_initial", (DL_FUNC) &mer_initial, 1},
     {"mer_isNested", (DL_FUNC) &mer_isNested, 1},
     {"mer_postVar", (DL_FUNC) &mer_postVar, 1},
+    {"mer2_postVar", (DL_FUNC) &mer2_postVar, 1},
     {"mer_ranef", (DL_FUNC) &mer_ranef, 1},
+    {"mer2_ranef", (DL_FUNC) &mer2_ranef, 1},
     {"mer_secondary", (DL_FUNC) &mer_secondary, 1},
     {"mer2_setPars", (DL_FUNC) &mer2_setPars, 2},
     {"mer_sigma", (DL_FUNC) &mer_sigma, 2},
