@@ -1670,7 +1670,35 @@ SEXP Zt_carryOver(SEXP fp, SEXP Zt, SEXP tvar, SEXP discount)
     return M_chm_sparse_to_SEXP(ans, 1, 0, 0, "", R_NilValue);
 }
 
+static void
+internal_TS_multiply(int nf, const int *nc, const int *Gp,
+		     double **ST, double *b)
+{
+    int i, ione = 1;
+    for (i = 0; i < nf; i++) {
+	int j, k, nci = nc[i], ncip1 = nc[i] + 1,
+	    nlev = (Gp[i + 1] - Gp[i])/nc[i];
 
+	for (j = 0; j < nlev; j++) {
+	    int base = Gp[i] + j * nci;
+	    for (k = 0; k < nci; k++) /* multiply by S_i */
+		b[base + k] *= ST[i][k * ncip1];
+	    if (nci > 1) 	/* multiply by T_i */
+		F77_CALL(dtrmv)("L", "N", "U", &nci, ST[i], &nci,
+				b + base, &ione);
+	}
+    }
+}
+
+/**
+ * Evaluate the effects in an mer2 representation
+ *
+ * @param L factorization
+ * @param dims vector of dimensions
+ * @param fixef vector to be filled with the fixed effects
+ * @param ranef vector to be filled with the random effects
+ *
+ */
 static void
 internal_mer2_effects(const cholmod_factor *L, const int *dims,
 		      double *fixef, double *ranef)
@@ -1695,16 +1723,34 @@ internal_mer2_effects(const cholmod_factor *L, const int *dims,
 	error(_("cholmod_solve (CHOLMOD_Lt) failed: status %d, minor %d from ncol %d"),
 	      c.status, L->minor, L->n);
     M_cholmod_free_dense(&B, &c);
-    if (!(B = M_cholmod_solve(CHOLMOD_P, L, X, &c)))
-	error(_("cholmod_solve (CHOLMOD_P) failed: status %d, minor %d from ncol %d"),
+    if (!(B = M_cholmod_solve(CHOLMOD_Pt, L, X, &c)))
+	error(_("cholmod_solve (CHOLMOD_Pt) failed: status %d, minor %d from ncol %d"),
 	      c.status, L->minor, L->n);
     M_cholmod_free_dense(&X, &c);
     Memcpy(ranef, (double*)(B->x), q);
     Memcpy(fixef, (double*)(B->x) + q, p);
     M_cholmod_free_dense(&B, &c);
+    /* We now have b* in the random effects slot.  We need to create
+     * b = STb*.
+     * */
 }
 
-static void
+/**
+ * Evaluate the logarithm of the square of the determinant of selected
+ * sections of a sparse Cholesky factor.
+ *
+ * @param ans vector of doubles of sufficient length to hold the result
+ * @param nans number of values to calculate
+ * @param c vector of length nans+1 containing the cut points
+
+FIXME: It seems that there is an implicit assumtion that c[0] = 0 but
+this is never checked.
+
+ * @param F factorization
+ *
+ * @return ans
+ */
+static double*
 chm_log_abs_det2(double *ans, int nans, const int *c, const cholmod_factor *F)
 {
     int i, ii  = 0, jj = 0;
@@ -1734,9 +1780,20 @@ chm_log_abs_det2(double *ans, int nans, const int *c, const cholmod_factor *F)
 	    ans[ii] += log(fx[k] * ((F->is_ll) ? fx[k] : 1.));
 	}
     }
+    return ans;
 }
 
-static void
+/**
+ * Evaluate the elements of the deviance slot given a factorization of
+ * A* and the dimensions vector.
+ *
+ * @param d pointer to the contents of the slot
+ * @param dims dimensions
+ * @param L factor of the current A*
+ *
+ * @return d
+ */
+static double*
 internal_deviance(double *d, const int *dims, const cholmod_factor *L)
 {
     int n = dims[n_POS], p = dims[p_POS], q = dims[q_POS];
@@ -1746,6 +1803,7 @@ internal_deviance(double *d, const int *dims, const cholmod_factor *L)
     chm_log_abs_det2(d + 2, 3, c, L);
     d[0] = d[2] + dn * (1. + d[4] + log(2. * PI / dn));
     d[1] = d[2] + d[3] + dnmp * (1. + d[4] + log(2. * PI / dnmp));
+    return d;
 }
 
 /**
@@ -1943,7 +2001,7 @@ SEXP mer2_create(SEXP fl, SEXP ZZt, SEXP Xtp, SEXP yp, SEXP REMLp,
     cholmod_dense *Xy;
     cholmod_factor *L;
     int *Perm, *Gp, *nc = INTEGER(ncp), *dims, *xdims, *zdims,
-	i, j, nf = LENGTH(fl), nobs = LENGTH(yp), p, q;
+	i, j, k, nf = LENGTH(fl), nobs = LENGTH(yp), p, q;
     double **st = Calloc(nf, double*), *Xt, *fixef, *offv,
 	*ranef, *wtv, *y;
 				/* record dimensions */
@@ -2053,11 +2111,14 @@ SEXP mer2_create(SEXP fl, SEXP ZZt, SEXP Xtp, SEXP yp, SEXP REMLp,
     internal_mer2_initial(st, nf, nc, Gp, ts2);
     i = c.nmethods;
     c.nmethods = 1;		/* force user-specified permutation */
+    j = c.postorder;
+    c.postorder = FALSE;
     				/* Create L  with user-specified perm */
     L = M_cholmod_analyze_p(ts2, Perm, (int*)NULL, (size_t)0, &c);
     if (!L)
 	error(_("cholmod_analyze_p returned with status %d"), c.status);
     c.nmethods = i;
+    c.postorder = j;
     				/* initialize and store L */
     SET_SLOT(val, lme4_devianceSym,
 	     internal_make_named(REALSXP, DEVIANCE_NAMES));
@@ -2172,13 +2233,18 @@ SEXP mer2_deviance(SEXP x, SEXP which)
  */
 SEXP mer2_update_effects(SEXP x)
 {
+    SEXP ST = GET_SLOT(x, lme4_STSym);
+    int i, nf = LENGTH(ST);
+    double *b = REAL(GET_SLOT(x, lme4_ranefSym));
+    double **st = Calloc(nf, double*);
     cholmod_factor *L = M_as_cholmod_factor(GET_SLOT(x, lme4_LSym));
+
     internal_mer2_effects(L, INTEGER(GET_SLOT(x, lme4_dimsSym)),
-			  REAL(GET_SLOT(x, lme4_fixefSym)),
-			  REAL(GET_SLOT(x, lme4_ranefSym)));
-    /* FIXME: Are the contents of the ranef slot the b*'s or the b's?  */
-    /* They are the b*'s. They should be multiplied by S. */
-    Free(L);
+			  REAL(GET_SLOT(x, lme4_fixefSym)), b);
+    for (i = 0; i < nf; i++) st[i] = REAL(VECTOR_ELT(ST, i));
+    internal_TS_multiply(nf, INTEGER(GET_SLOT(x, lme4_ncSym)),
+			 INTEGER(GET_SLOT(x, lme4_GpSym)), st, b);
+    Free(L); Free(st);
     return R_NilValue;
 }
 
@@ -2250,6 +2316,7 @@ SEXP mer2_vcov(SEXP x)
 	M_cholmod_free_sparse(&Lred, &c);
 	Memcpy(REAL(ans), (double*)(Ld->x), p * p);
 	M_cholmod_free_dense(&Ld, &c);
+/* FIXME: This does not allow for a possible P_X permutation  */
 	F77_CALL(dtrtri)("L", "N", &p, REAL(ans), &p, &i);
 	if (i)
 	    error(_("Lapack routine dtrtri returned error code %d"), i);
@@ -2259,3 +2326,145 @@ SEXP mer2_vcov(SEXP x)
     return ans;
 }
 
+/**
+ * Extract the conditional modes of the random effects as a list of matrices
+ *
+ * @param x Pointer to an mer object
+ *
+ * @return a list of matrices containing the conditional modes of the
+ * random effects
+ */
+SEXP mer2_ranef(SEXP x)
+{
+    SEXP cnames = GET_SLOT(x, lme4_cnamesSym),
+	flist = GET_SLOT(x, lme4_flistSym);
+    int *Gp = INTEGER(GET_SLOT(x, lme4_GpSym)),
+	*nc = INTEGER(GET_SLOT(x, lme4_ncSym)),
+	i, ii, jj,
+	nf = LENGTH(flist);
+    SEXP val = PROTECT(allocVector(VECSXP, nf));
+    double *b = REAL(GET_SLOT(x, lme4_ranefSym));
+
+    mer2_update_effects(x);
+    setAttrib(val, R_NamesSymbol,
+	      duplicate(getAttrib(flist, R_NamesSymbol)));
+    for (i = 0; i < nf; i++) {
+	SEXP nms, rnms = getAttrib(VECTOR_ELT(flist, i), R_LevelsSymbol);
+	int nci = nc[i], mi = length(rnms);
+	double *bi = b + Gp[i], *mm;
+
+	SET_VECTOR_ELT(val, i, allocMatrix(REALSXP, mi, nci));
+	setAttrib(VECTOR_ELT(val, i), R_DimNamesSymbol, allocVector(VECSXP, 2));
+	nms = getAttrib(VECTOR_ELT(val, i), R_DimNamesSymbol);
+	SET_VECTOR_ELT(nms, 0, duplicate(rnms));
+	SET_VECTOR_ELT(nms, 1, duplicate(VECTOR_ELT(cnames, i)));
+	mm = REAL(VECTOR_ELT(val, i));
+	for (jj = 0; jj < nci; jj++)
+	    for(ii = 0; ii < mi; ii++)
+		mm[ii + jj * mi] = bi[jj + ii * nci];
+    }
+    UNPROTECT(1);
+    return val;
+}
+
+/**
+ * Extract the posterior variances of the random effects
+ *
+ * @param x pointer to a mer object
+ *
+ * @return pointer to a list of arrays
+ */
+SEXP mer2_postVar(SEXP x)
+{
+    double *deviance = REAL(GET_SLOT(x, lme4_devianceSym)), one = 1;
+    int *Gp = INTEGER(GET_SLOT(x, lme4_GpSym)),
+	*dims = INTEGER(GET_SLOT(x, lme4_dimsSym)),
+	*nc = INTEGER(GET_SLOT(x, lme4_ncSym));
+    int i, j, nf = dims[nf_POS], p = dims[p_POS], q = dims[q_POS];
+    int ppq = p + q;
+    double sc = internal_mer2_sigma(isREML(x), dims, deviance);
+    cholmod_factor *L = M_as_cholmod_factor(GET_SLOT(x, lme4_LSym)),
+	*Lcp = (cholmod_factor*)NULL;
+    cholmod_sparse *rhs, *B, *Bt, *BtB;
+    cholmod_dense *BtBd;
+    int *Perm = (int*)(L->Perm), *iperm = Calloc(ppq, int),
+	*fset = Calloc(ppq, int);
+    SEXP ST = GET_SLOT(x, lme4_STSym),
+	ans = PROTECT(allocVector(VECSXP, nf));
+    
+    for (j = 0; j < ppq; j++) {
+	iperm[Perm[j]] = j;
+	fset[j] = j;
+    }
+    if (!L->is_ll) {
+	Lcp = M_cholmod_copy_factor(L, &c);
+	Free(L);
+	L = Lcp;
+	j = M_cholmod_change_factor(CHOLMOD_REAL, TRUE/*ll*/,
+				    FALSE/*super*/, TRUE/*packed*/,
+				    TRUE/*sorted*/, L, &c);
+	if (!j) error(_("cholmod_change_factor failed"));
+    }
+    sc = sc * sc;		/* variance scale factor */
+    for (i = 0; i < nf; i++) {
+	int j, k, kk, nci = nc[i], nlev = (Gp[i + 1] - Gp[i])/nc[i];
+	SEXP ansi = PROTECT(alloc3Darray(REALSXP, nci, nci, nlev));
+	int ncip1 = nci + 1, ncisqr = nci * nci;
+	double *vv = REAL(ansi),
+	    *st = Memcpy(Calloc(ncisqr, double),
+			 REAL(VECTOR_ELT(ST, i)), ncisqr);
+
+	SET_VECTOR_ELT(ans, i, ansi); UNPROTECT(1);
+	AZERO(vv, ncisqr * nlev);
+	rhs = M_cholmod_allocate_sparse((size_t)(ppq + 1),
+					(size_t) nci, (size_t) nci,
+					1/*sorted*/, 1/*packed*/,
+					0/*stype*/, CHOLMOD_REAL, &c);
+	((int*)(rhs->p))[0] = 0;
+	for (k = 0; k < nci; k++) {
+	    ((double*)(rhs->x))[k] = 1.;
+	    ((int*)(rhs->p))[k + 1] = k + 1;
+	}
+	for (k = 0; k < nci; k++) {
+	    double mult = st[k * ncip1];
+	    for (kk = k + 1; kk < nci; kk++)
+		st[kk + k * nci] *= mult;
+	}
+	for (j = 0; j < nlev; j++) {
+	    int *ip, *pp, base = Gp[i] + j * nci;
+	    double *xp;
+	    
+	    for (k = 0; k < nci; k++)
+		((int*)(rhs->i))[k] = iperm[base + k];
+	    B = M_cholmod_spsolve(CHOLMOD_L, L, rhs, &c);
+	    ip = (int*)(B->i);
+	    pp = (int*)(B->p);
+	    xp = (double*)(B->x);
+	    if (nci == 1) {
+		for (k = 0; k < pp[1]; k++)
+		    if (ip[k] < ppq) vv[j] += xp[k] * xp[k];
+		vv[j] *= sc * st[0] * st[0];
+	    } else {
+		double *vvj = vv + j * ncisqr;
+		Bt = M_cholmod_transpose(B, TRUE/*values*/, &c);
+		BtB = M_cholmod_aat(Bt, fset, (size_t)ppq, 1/*mode*/,&c);
+		M_cholmod_free_sparse(&Bt, &c);
+		BtBd = M_cholmod_sparse_to_dense(BtB, &c);
+		M_cholmod_free_sparse(&BtB, &c);
+		Memcpy(vvj, (double*)(BtBd->x), ncisqr);
+		M_cholmod_free_dense(&BtBd, &c);
+		F77_CALL(dtrmm)("L", "L", "N", "N", &nci, &nci,
+				&one, st, &nci, vvj, &nci);
+		F77_CALL(dtrmm)("R", "L", "T", "N", &nci, &nci,
+				&sc, st, &nci, vvj, &nci);
+	    }
+	    M_cholmod_free_sparse(&B, &c);
+	}
+	M_cholmod_free_sparse(&rhs, &c);
+	Free(st);
+    }
+    if (L == Lcp) M_cholmod_free_factor(&L, &c); else Free(L);
+    Free(iperm); Free(fset);
+    UNPROTECT(1);
+    return ans;
+}
