@@ -48,7 +48,9 @@ SEXP mer_hat_trace2(SEXP x);
 SEXP mer_initial(SEXP x);
 SEXP mer_isNested(SEXP x);
 SEXP mer_postVar(SEXP x);
+SEXP mer2_postVar(SEXP x);
 SEXP mer_ranef(SEXP x);
+SEXP mer2_ranef(SEXP x);
 SEXP mer2_setPars(SEXP x, SEXP pars);
 SEXP mer_sigma(SEXP x, SEXP REML);
 SEXP mer2_sigma(SEXP x, SEXP which);
